#include <getopt.h>
#include <stdlib.h>
#include <signal.h>

#include "CUnit/Basic.h"
#include "CUnit/Console.h"
#include "CUnit/Automated.h"


#define VERSION "1.0.0" __DATE__ 

int cmdline_test_init();
int bigreq_test_init();
int simple_squid25_test_init();
int chunked_squid25_test_init();
int deadserver_test_init();
int protocol_parse_test_init();
int multiple_server_test_init();
int flapping_test_init();
int concurrent_test_init();
int performance_test_init();
int timeout_test_init();
int https_test_init();
int redir_squid_send_test_init();
int username_rewrite_test_init();
int append_domain_test_init();
int redir_fail_open_init();
int enormous_connections_test_init();
int simple_test_init();
int redir_input_output_test_init();

void usage(int argc, char** argv)
{
	if ( argc >= 1 ) {
		printf("%s: [OPTION]\n", argv[0]);
	}
	printf("OPTION can be one or more of the following\n");
	printf("\t-a to run all with output to xml\n");
	printf("\t-b for basic tests with stdout output\n");
	printf("\t-c for console to select specific tests\n");
	printf("\t-v for version\n");
	printf("\t-h for help\n");
}

int main(int argc, char** argv)
{
	int ret;
	char option;
	int valid_args=0;

	signal(SIGPIPE, SIG_IGN);

	/* initialize the CUnit test registry */
	if (CUE_SUCCESS != CU_initialize_registry()) {
		return CU_get_error();
	}

	ret=simple_squid25_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=chunked_squid25_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=deadserver_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=multiple_server_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=flapping_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=protocol_parse_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=concurrent_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=performance_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=redir_input_output_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=cmdline_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=timeout_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=https_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=redir_squid_send_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=username_rewrite_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=append_domain_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=redir_fail_open_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=bigreq_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=enormous_connections_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	ret=simple_test_init();
	if ( ret != 0 ) {
		return ret;
	}

	static struct option long_options[] = {
			{"automatic", 0, 0, 'a'},
			{"basic", 0, 0, 'b'},
			{"console", 0, 0, 'c'},
			{"version", 0, 0, 'v'},
			{"help", 0, 0, 'h'},
			{0, 0, 0, 0}
	};
	
	// use getopt to do the processing
	while ((option = getopt_long (argc, argv, "abcvh", long_options, 0)) != -1) {
		switch (option) {
			case 'c':
				/* Run all tests using the basic interface */
				CU_basic_set_mode(CU_BRM_VERBOSE);
				CU_console_run_tests();
				valid_args=1;
				break;
			case 'b':
				printf("\n\n");
				CU_basic_run_tests();
				printf("\n\n");
				CU_basic_show_failures(CU_get_failure_list());
				printf("\n\n");
				valid_args=1;
				break;
			case 'a':
				printf("\n\n");
				CU_automated_run_tests();
				CU_list_tests_to_file();
				printf("\n\n");
				valid_args=1;
			case 'v':
				printf("Netsweeper Redirector Tests " VERSION " Build: " __DATE__ " " __TIME__ "\n");
				valid_args=1;
				break;
			case 'h':
				usage(argc, argv);
				valid_args=1;
				break;
			case '?':
				usage(argc, argv);
				exit(1);
			default:
				usage(argc, argv);
				exit(1);
		}
	}

	if ( ! valid_args ) {
		usage(argc, argv);
		exit(1);
	}

	/* Clean up registry and return */
	CU_cleanup_registry();
	return CU_get_error();
}

